import { GraphQLSchema, IntrospectionQuery, IntrospectionField, IntrospectionInputValue } from 'graphql';
/**
 * Send introspection query to a graphql schema
 */
export declare const introspectSchema: (schema: GraphQLSchema) => Promise<IntrospectionQuery>;
export declare function introspect(schemaContents: string): Promise<IntrospectionQuery>;
export declare const introspectSchemaViaLocalFile: (path: string) => Promise<IntrospectionQuery>;
export interface SimpleTypeDescription {
    kind: string;
    name: string;
}
/**
 * Check if type is a built-in graphql type
 */
export declare const isBuiltinType: (type: SimpleTypeDescription) => boolean;
export interface GraphqlDescription {
    description?: string;
    isDeprecated?: boolean;
    deprecationReason?: string;
}
/**
 * Convert description and deprecated directives into JSDoc
 */
export declare const descriptionToJSDoc: (description: GraphqlDescription) => string[];
export interface FieldType {
    fieldModifier: string;
    refName: string;
    refKind: string;
}
export declare const getFieldRef: (field: IntrospectionField | IntrospectionInputValue) => FieldType;
export declare const formatTabSpace: (lines: string[], tabSpaces: number) => string[];
export declare const createFieldRef: (fieldName: string, refName: string, fieldModifier: string) => string;
export declare const gqlScalarToTS: (scalarName: string, typePrefix: string) => string;
export declare const toUppercaseFirst: (value: string) => string;
